import uuid

from datetime import datetime
from decimal import Decimal
from typing import TYPE_CHECKING

from sqlalchemy import ForeignKey, Numeric, UniqueConstraint, Uuid, text
from sqlalchemy.orm import Mapped, mapped_column, relationship

from ..db.base_class import reg


if TYPE_CHECKING:
    from .customer import Customer
    from .sale_category import SaleCategory


@reg.mapped_as_dataclass(unsafe_hash=True)
class CustomerDiscount:
    __tablename__ = "customer_discount"
    __table_args__ = (UniqueConstraint("customer_id", "sale_category_id"),)

    id: Mapped[uuid.UUID] = mapped_column(Uuid, primary_key=True, server_default=text("gen_random_uuid()"))
    customer_id: Mapped[uuid.UUID] = mapped_column(Uuid, ForeignKey("customers.id"), nullable=False)
    sale_category_id: Mapped[uuid.UUID] = mapped_column(Uuid, ForeignKey("sale_categories.id"), nullable=False)
    discount: Mapped[Decimal] = mapped_column(Numeric(precision=15, scale=5), nullable=False)

    customer: Mapped["Customer"] = relationship(back_populates="discounts")
    sale_category: Mapped["SaleCategory"] = relationship("SaleCategory")

    def __init__(self, sale_category_id=None, discount=None, id_=None, customer_id=None, customer=None):
        self.sale_category_id = sale_category_id
        self.discount = discount
        self.id = id_
        self.date = datetime.utcnow()
        if customer is None:
            self.customer_id = customer_id
        else:
            self.customer = customer
