import uuid

from decimal import Decimal
from typing import TYPE_CHECKING

from sqlalchemy import Boolean, ForeignKey, Numeric, Text, Uuid, text
from sqlalchemy.orm import Mapped, mapped_column, relationship

from ..db.base_class import reg


if TYPE_CHECKING:
    from .modifier_category import ModifierCategory


@reg.mapped_as_dataclass(unsafe_hash=True)
class Modifier:
    __tablename__ = "modifiers"

    id: Mapped[uuid.UUID] = mapped_column(Uuid, primary_key=True, server_default=text("gen_random_uuid()"))
    name: Mapped[str] = mapped_column(Text, nullable=False, unique=True)
    show_in_bill: Mapped[bool] = mapped_column(Boolean, nullable=False)
    price: Mapped[Decimal] = mapped_column(Numeric(precision=15, scale=2), nullable=False)
    is_active: Mapped[bool] = mapped_column(Boolean, nullable=False)
    modifier_category_id: Mapped[uuid.UUID] = mapped_column(
        Uuid,
        ForeignKey("modifier_categories.id"),
        nullable=False,
    )

    modifier_category: Mapped["ModifierCategory"] = relationship(back_populates="modifiers")

    def __init__(
        self,
        name=None,
        show_in_bill=None,
        price=None,
        modifier_category_id=None,
        is_active=True,
        id_=None,
    ):
        self.id = id_
        self.name = name
        self.show_in_bill = show_in_bill
        self.price = price
        self.modifier_category_id = modifier_category_id
        self.is_active = is_active
