from __future__ import annotations

import uuid

from typing import TYPE_CHECKING

from barker.models.overview_status import OverviewStatus
from sqlalchemy import Enum, ForeignKey, Uuid, text
from sqlalchemy.orm import Mapped, mapped_column, relationship

from ..db.base_class import reg


if TYPE_CHECKING:
    from .food_table import FoodTable
    from .guest_book import GuestBook
    from .voucher import Voucher


@reg.mapped_as_dataclass(unsafe_hash=True)
class Overview:
    __tablename__ = "overview"
    id: Mapped[uuid.UUID] = mapped_column(Uuid, primary_key=True, server_default=text("gen_random_uuid()"))
    voucher_id: Mapped[uuid.UUID] = mapped_column(
        Uuid,
        ForeignKey("vouchers.id"),
        nullable=False,
        unique=True,
    )
    food_table_id: Mapped[uuid.UUID] = mapped_column(
        Uuid,
        ForeignKey("food_tables.id"),
        nullable=False,
        unique=True,
    )
    guest_book_id: Mapped[uuid.UUID] = mapped_column(Uuid, ForeignKey("guest_book.id"), unique=True, nullable=True)
    status: Mapped[OverviewStatus] = mapped_column(Enum(OverviewStatus), nullable=False)

    voucher: Mapped["Voucher"] = relationship(back_populates="status")
    food_table: Mapped["FoodTable"] = relationship(back_populates="status")
    guest: Mapped["GuestBook"] = relationship(back_populates="status")

    def __init__(self, voucher_id, food_table_id, guest_book_id, status, id_=None):
        self.voucher_id = voucher_id
        self.food_table_id = food_table_id
        self.guest_book_id = guest_book_id
        self.status = status
        self.id = id_
