import uuid

from decimal import Decimal
from typing import TYPE_CHECKING, List

from sqlalchemy import ForeignKey, Numeric, Text, Uuid, text
from sqlalchemy.orm import Mapped, mapped_column, relationship

from ..db.base_class import reg


if TYPE_CHECKING:
    from .product_version import ProductVersion
    from .tax import Tax


@reg.mapped_as_dataclass(unsafe_hash=True)
class SaleCategory:
    __tablename__ = "sale_categories"

    id: Mapped[uuid.UUID] = mapped_column(Uuid, primary_key=True, server_default=text("gen_random_uuid()"))
    name: Mapped[str] = mapped_column(Text, nullable=False, unique=True)
    discount_limit: Mapped[Decimal] = mapped_column(Numeric(precision=15, scale=5), nullable=False)
    tax_id: Mapped[uuid.UUID] = mapped_column(Uuid, ForeignKey("taxes.id"), nullable=False)

    tax: Mapped["Tax"] = relationship(back_populates="sale_categories")
    products: Mapped[List["ProductVersion"]] = relationship(back_populates="sale_category")

    def __init__(self, name, discount_limit=None, tax_id=False, id_=None):
        self.name = name
        self.discount_limit = discount_limit if discount_limit is not None else 1
        self.tax_id = tax_id
        self.id = id_

    @classmethod
    def cash_charges(cls):
        return uuid.UUID("f3d6441a-7018-4454-9704-6b99db9333c7")
