import uuid

from barker.schemas.regime import RegimeLink
from pydantic import BaseModel, ConfigDict, Field

from . import Daf, to_camel


class TaxIn(BaseModel):
    name: str = Field(..., min_length=1)
    rate: Daf = Field(ge=0, default=0)
    regime: RegimeLink = Field(...)
    is_fixture: bool
    model_config = ConfigDict(str_strip_whitespace=True, alias_generator=to_camel, populate_by_name=True)


class Tax(TaxIn):
    id_: uuid.UUID
    model_config = ConfigDict(str_strip_whitespace=True, alias_generator=to_camel, populate_by_name=True)


class TaxBlank(BaseModel):
    name: str
    rate: Daf = Field(ge=0, default=0)
    is_fixture: bool
    model_config = ConfigDict(str_strip_whitespace=True, alias_generator=to_camel, populate_by_name=True)


class TaxLink(BaseModel):
    id_: uuid.UUID = Field(...)
    name: str | None = None
    rate: Daf | None = None
    model_config = ConfigDict(alias_generator=to_camel, populate_by_name=True)
