import uuid

from datetime import date, datetime

from pydantic import BaseModel, ConfigDict, field_serializer, field_validator

from . import Daf, to_camel


class UpdateProductPricesItem(BaseModel):
    id: uuid.UUID
    name: str
    old_price: Daf
    new_price: Daf
    model_config = ConfigDict(alias_generator=to_camel, populate_by_name=True)


class UpdateProductPrices(BaseModel):
    date_: date
    menu_category_id: uuid.UUID | None = None
    items: list[UpdateProductPricesItem]
    model_config = ConfigDict(alias_generator=to_camel, populate_by_name=True)

    @field_validator("date_", mode="before")
    @classmethod
    def parse_date(cls, value):
        if isinstance(value, date):
            return value
        return datetime.strptime(value, "%d-%b-%Y").date()

    @field_serializer("date_")
    def serialize_date(self, value: date, _info):
        return value.strftime("%d-%b-%Y")
